@@ -1,9 +1,11 @@
+mod data_store_base_py;
 mod net_provider_py;
 mod observability_client_base_py;
 mod pyo_utils;
 mod statsig_base_py;
 mod statsig_metadata_py;
 mod statsig_options_py;
+mod statsig_persistent_storage_override_adapter_py;
 mod statsig_types_py;
 mod statsig_user_py;
 
@@ -26,6 +28,8 @@ fn statsig_python_core(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_class::<statsig_types_py::DynamicConfigEvaluationOptionsPy>()?;
     m.add_class::<statsig_types_py::LayerEvaluationOptionsPy>()?;
     m.add_class::<observability_client_base_py::ObservabilityClientBasePy>()?;
+    m.add_class::<data_store_base_py::DataStoreBasePy>()?;
+    m.add_class::<statsig_persistent_storage_override_adapter_py::PersistentStorageBasePy>()?;
     Ok(())
 }
 
