@@ -14,6 +14,7 @@ pub use override_adapter::{
     override_adapter_trait::OverrideAdapter,
     statsig_local_override_adapter::StatsigLocalOverrideAdapter,
 };
+pub use persistent_storage::persistent_storage_trait::*;
 pub use sampling_processor::SamplingProcessor;
 pub use spec_store::SpecStore;
 pub use specs_adapter::*;
@@ -25,10 +26,11 @@ pub use statsig_core_api_options::{
 pub use statsig_err::StatsigErr;
 pub use statsig_options::StatsigOptions;
 pub use statsig_runtime::StatsigRuntime;
-pub use statsig_user::StatsigUser;
+pub use user::{StatsigUser, StatsigUserBuilder};
 
 pub mod compression;
 pub mod data_store_interface;
+pub mod evaluation;
 pub mod global_configs;
 pub mod hashing;
 pub mod hashset_with_ttl;
@@ -43,17 +45,17 @@ pub mod statsig_core_api_options;
 pub mod statsig_metadata;
 pub mod statsig_options;
 pub mod statsig_types;
-pub mod statsig_user;
+pub mod user;
 
 mod client_init_response_formatter;
 mod dcs_str;
-mod evaluation;
 mod event_logging;
 mod event_logging_adapter;
 mod id_lists_adapter;
 mod initialize_response;
 mod macros;
 mod observability;
+mod persistent_storage;
 mod sampling_processor;
 mod spec_store;
 mod specs_adapter;
@@ -61,4 +63,3 @@ mod statsig;
 mod statsig_err;
 mod statsig_runtime;
 mod statsig_type_factories;
-mod statsig_user_internal;
