@@ -1,7 +1,7 @@
 use async_trait::async_trait;
 
 use crate::{
-    networking::{HttpMethod, NetProviderRequestArgs, NetworkProvider, Response},
+    networking::{HttpMethod, NetworkProvider, RequestArgs, Response},
     StatsigErr,
 };
 
@@ -9,7 +9,7 @@ pub struct NetworkProviderNoop;
 
 #[async_trait]
 impl NetworkProvider for NetworkProviderNoop {
-    async fn send(&self, _method: &HttpMethod, _request_args: &NetProviderRequestArgs) -> Response {
+    async fn send(&self, _method: &HttpMethod, _request_args: &RequestArgs) -> Response {
         Response {
             status_code: 0,
             data: None,
