@@ -1,3 +1,5 @@
+use std::sync::RwLock;
+
 use log::{debug, error, info, warn, Level};
 
 const MAX_CHARS: usize = 400;
@@ -5,6 +7,10 @@ const TRUNCATED_SUFFIX: &str = "...[TRUNCATED]";
 
 const DEFAULT_LOG_LEVEL: LogLevel = LogLevel::Warn;
 
+lazy_static::lazy_static! {
+    static ref LOG_LEVEL: RwLock<LogLevel> = RwLock::new(DEFAULT_LOG_LEVEL);
+}
+
 #[derive(Clone)]
 pub enum LogLevel {
     None,
@@ -50,11 +56,25 @@ impl LogLevel {
             LogLevel::None => None,
         }
     }
+
+    fn to_number(&self) -> u32 {
+        match self {
+            LogLevel::Debug => 4,
+            LogLevel::Info => 3,
+            LogLevel::Warn => 2,
+            LogLevel::Error => 1,
+            LogLevel::None => 0,
+        }
+    }
 }
 
 pub fn initialize_simple_output_logger(level: &Option<LogLevel>) {
     let level = level.as_ref().unwrap_or(&DEFAULT_LOG_LEVEL).clone();
 
+    if let Ok(mut lock) = LOG_LEVEL.write() {
+        *lock = level.clone();
+    }
+
     let final_level = match level {
         LogLevel::None => {
             return;
@@ -119,31 +139,60 @@ fn sanitize(input: &str) -> String {
         .collect()
 }
 
+pub fn has_valid_log_level(level: &LogLevel) -> bool {
+    let current_level = match LOG_LEVEL.read() {
+        Ok(lock) => lock,
+        Err(_) => return false,
+    };
+
+    level.to_number() <= current_level.to_number()
+}
+
 #[macro_export]
 macro_rules! log_d {
   ($tag:expr, $($arg:tt)*) => {
-        $crate::output_logger::log_message($tag, $crate::output_logger::LogLevel::Debug, format!($($arg)*))
+        {
+            let level = $crate::output_logger::LogLevel::Debug;
+            if $crate::output_logger::has_valid_log_level(&level) {
+                $crate::output_logger::log_message($tag, level, format!($($arg)*));
+            }
+        }
     }
 }
 
 #[macro_export]
 macro_rules! log_i {
   ($tag:expr, $($arg:tt)*) => {
-        $crate::output_logger::log_message($tag, $crate::output_logger::LogLevel::Info, format!($($arg)*))
+        {
+            let level = $crate::output_logger::LogLevel::Info;
+            if $crate::output_logger::has_valid_log_level(&level) {
+                $crate::output_logger::log_message($tag, level, format!($($arg)*));
+            }
+        }
     }
 }
 
 #[macro_export]
 macro_rules! log_w {
   ($tag:expr, $($arg:tt)*) => {
-        $crate::output_logger::log_message($tag, $crate::output_logger::LogLevel::Warn, format!($($arg)*))
+        {
+            let level = $crate::output_logger::LogLevel::Warn;
+            if $crate::output_logger::has_valid_log_level(&level) {
+                $crate::output_logger::log_message($tag, level, format!($($arg)*));
+            }
+        }
     }
 }
 
 #[macro_export]
 macro_rules! log_e {
   ($tag:expr, $($arg:tt)*) => {
-        $crate::output_logger::log_message($tag, $crate::output_logger::LogLevel::Error, format!($($arg)*))
+        {
+            let level = $crate::output_logger::LogLevel::Error;
+            if $crate::output_logger::has_valid_log_level(&level) {
+                $crate::output_logger::log_message($tag, level, format!($($arg)*));
+            }
+        }
     }
 }
 
@@ -152,9 +201,11 @@ macro_rules! log_error_to_statsig_and_console {
   ($ops_stats:expr, $tag:expr, $($arg:tt)*) => {
     let err_message = format!($($arg)*);
     let event = ErrorBoundaryEvent {
+        bypass_dedupe: false,
         exception: err_message.clone(),
         tag: $tag.to_string(),
         extra: None,
+        dedupe_key: None
     };
     $ops_stats.log_error(event);
 
