@@ -1,3 +1,4 @@
+use crate::compression::zstd_decompression_dict::DictionaryDecoder;
 use crate::data_store_interface::{get_data_adapter_dcs_key, DataStoreTrait};
 use crate::global_configs::GlobalConfigs;
 use crate::id_lists_adapter::{IdList, IdListsUpdateListener};
@@ -19,7 +20,7 @@ pub struct SpecStoreData {
     pub source: SpecsSource,
     pub time_received_at: Option<u64>,
     pub values: SpecsResponseFull,
-
+    pub decompression_dict: Option<DictionaryDecoder>,
     pub id_lists: HashMap<String, IdList>,
 }
 
@@ -44,6 +45,10 @@ impl SpecsUpdateListener for SpecStore {
             Ok(data) => SpecsInfo {
                 lcut: Some(data.values.time),
                 checksum: data.values.checksum.clone(),
+                zstd_dict_id: data
+                    .decompression_dict
+                    .as_ref()
+                    .map(|d| d.get_dict_id().to_string()),
                 source: data.source.clone(),
             },
             Err(e) => {
@@ -51,6 +56,7 @@ impl SpecsUpdateListener for SpecStore {
                 SpecsInfo {
                     lcut: None,
                     checksum: None,
+                    zstd_dict_id: None,
                     source: SpecsSource::Error,
                 }
             }
@@ -125,6 +131,7 @@ impl SpecStore {
                 values: SpecsResponseFull::blank(),
                 time_received_at: None,
                 source: SpecsSource::Uninitialized,
+                decompression_dict: None,
                 id_lists: HashMap::new(),
             })),
             data_store,
