@@ -82,10 +82,10 @@ impl StatsigHttpSpecsAdapter {
             query_params: Some(params),
             accept_gzip_response: true,
             diagnostics_key: Some(KeyType::DownloadConfigSpecs),
-            ..RequestArgs::default()
+            ..RequestArgs::new()
         };
 
-        match self.network.get(request_args.clone()).await {
+        match self.handle_specs_request(request_args.clone()).await {
             Ok(response) => Ok(response),
             Err(NetworkError::RetriesExhausted) => self.handle_fallback_request(request_args).await,
             Err(e) => Err(e),
@@ -105,7 +105,20 @@ impl StatsigHttpSpecsAdapter {
 
         // TODO logging
 
-        self.network.get(request_args).await
+        self.handle_specs_request(request_args).await
+    }
+
+    // TODO: return a decompressed and parsed SpecsResponse
+    async fn handle_specs_request(
+        &self,
+        request_args: RequestArgs,
+    ) -> Result<String, NetworkError> {
+        let response = self.network.get(request_args).await?;
+        match response.data {
+            Some(data) => Ok(String::from_utf8(data)
+                .map_err(|e| NetworkError::SerializationError(e.to_string()))?),
+            None => Err(NetworkError::RequestFailed),
+        }
     }
 
     pub async fn run_background_sync(weak_self: &Weak<Self>) {
@@ -128,6 +141,9 @@ impl StatsigHttpSpecsAdapter {
             .ops_stats
             .set_diagnostics_context(ContextType::ConfigSync);
         if let Err(e) = strong_self.manually_sync_specs(specs_info).await {
+            if let StatsigErr::NetworkError(NetworkError::DisableNetworkOn, _) = e {
+                return;
+            }
             log_e!(TAG, "Background specs sync failed: {}", e);
         }
         strong_self.ops_stats.enqueue_diagnostics_event(
@@ -147,7 +163,6 @@ impl StatsigHttpSpecsAdapter {
 
         let data = response.map_err(|e| {
             let msg = "No specs result from network";
-            log_e!(TAG, "{}", msg);
             StatsigErr::NetworkError(e, Some(msg.to_string()))
         })?;
 
@@ -276,7 +291,7 @@ fn construct_specs_url(sdk_key: &str, spec_url: Option<&String>) -> String {
 // only fallback when the spec_url is not the DEFAULT_SPECS_URL
 fn construct_fallback_specs_url(sdk_key: &str, spec_url: Option<&String>) -> Option<String> {
     match spec_url {
-        Some(u) if u != DEFAULT_SPECS_URL => Some(format!("{u}/{sdk_key}.json")),
+        Some(u) if u != DEFAULT_SPECS_URL => Some(format!("{DEFAULT_SPECS_URL}/{sdk_key}.json")),
         _ => None,
     }
 }
