@@ -28,6 +28,7 @@ use crate::observability::observability_client_adapter::{MetricType, Observabili
 use crate::observability::ops_stats::{OpsStatsForInstance, OPS_STATS};
 use crate::observability::sdk_errors_observer::{ErrorBoundaryEvent, SDKErrorsObserver};
 use crate::output_logger::initialize_simple_output_logger;
+use crate::persistent_storage::persistent_values_manager::PersistentValuesManager;
 use crate::sdk_diagnostics::diagnostics::{ContextType, Diagnostics};
 use crate::sdk_diagnostics::marker::{ActionType, KeyType, Marker, StepType};
 use crate::spec_store::{SpecStore, SpecStoreData};
@@ -42,7 +43,7 @@ use crate::statsig_type_factories::{
 use crate::statsig_types::{
     DynamicConfig, Experiment, FeatureGate, Layer, OverrideAdapterType, ParameterStore,
 };
-use crate::statsig_user_internal::StatsigUserInternal;
+use crate::user::StatsigUserInternal;
 use crate::{
     dyn_value, log_d, log_e, log_w, read_lock_or_else, IdListsAdapter, ObservabilityClient,
     OpsStatsEventObserver, OverrideAdapter, SamplingProcessor, SpecsAdapter, SpecsInfo,
@@ -60,13 +61,17 @@ use serde::de::DeserializeOwned;
 use serde_json::json;
 use serde_json::Value;
 use std::collections::HashMap;
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Mutex;
 use std::sync::{Arc, Weak};
 use std::time::{Duration, Instant};
+use tokio::time::sleep;
 use tokio::try_join;
 
 const TAG: &str = stringify!(Statsig);
 const ERROR_SDK_KEY: &str = "__STATSIG_ERROR_SDK_KEY__";
+const INIT_IP_TAG: &str = "INIT_COUNTRY_LOOKUP";
+const INIT_UA_TAG: &str = "INIT_UA";
 
 lazy_static::lazy_static! {
     static ref SHARED_INSTANCE: Mutex<Option<Arc<Statsig>>> = Mutex::new(None);
@@ -91,6 +96,8 @@ pub struct Statsig {
     error_observer: Arc<dyn OpsStatsEventObserver>,
     diagnostics_observer: Arc<dyn OpsStatsEventObserver>,
     sampling_processor: Arc<SamplingProcessor>,
+    background_tasks_started: Arc<AtomicBool>,
+    persistent_values_manager: Option<Arc<PersistentValuesManager>>,
 }
 
 pub struct StatsigContext {
@@ -163,14 +170,6 @@ impl Statsig {
             Some(statsig_runtime.clone()),
         ));
 
-        if options.enable_user_agent_parsing.unwrap_or(false) {
-            UserAgentParser::load_parser();
-        }
-
-        if options.enable_country_lookup.unwrap_or(false) {
-            CountryLookup::load_country_lookup();
-        }
-
         let environment = options
             .environment
             .as_ref()
@@ -182,6 +181,12 @@ impl Statsig {
             sdk_key,
         ));
 
+        let persistent_values_manager = options.persistent_storage.clone().map(|storage| {
+            Arc::new(PersistentValuesManager {
+                persistent_storage: storage,
+            })
+        });
+
         StatsigMetadata::update_service_name(options.service_name.clone());
 
         Statsig {
@@ -205,124 +210,42 @@ impl Statsig {
             error_observer,
             sampling_processor,
             diagnostics_observer,
+            background_tasks_started: Arc::new(AtomicBool::new(false)),
+            persistent_values_manager,
         }
     }
 
     pub async fn initialize(&self) -> Result<(), StatsigErr> {
-        self.initialize_with_details().await.map(|_| ())
-    }
-
-    pub async fn initialize_with_details(&self) -> Result<StatsigInitializeDetails, StatsigErr> {
-        let start_time = Instant::now();
         self.ops_stats.add_marker(
             Marker::new(KeyType::Overall, ActionType::Start, None),
             Some(ContextType::Initialize),
         );
-        self.spec_store.set_source(SpecsSource::Loading);
-        self.event_logger
-            .clone()
-            .start_background_task(&self.statsig_runtime);
 
-        self.specs_adapter.initialize(self.spec_store.clone());
-
-        let mut success = true;
-        let mut error_message = None;
-        let mut id_list_ready = None;
-
-        let init_res = match self
-            .specs_adapter
-            .clone()
-            .start(&self.statsig_runtime)
-            .await
-        {
-            Ok(()) => Ok(()),
-            Err(e) => {
-                self.spec_store.set_source(SpecsSource::NoValues);
-                error_message = Some(format!("Failed to start specs adapter: {e}"));
-                Err(e)
-            }
+        let init_details = if let Some(timeout_ms) = self.options.init_timeout_ms {
+            self.apply_timeout_to_init(timeout_ms).await
+        } else {
+            self.initialize_impl_with_details().await
         };
 
-        if let Some(adapter) = &self.id_lists_adapter {
-            match adapter
-                .clone()
-                .start(&self.statsig_runtime, self.spec_store.clone())
-                .await
-            {
-                Ok(()) => {
-                    id_list_ready = Some(true);
-                }
-                Err(e) => {
-                    id_list_ready = Some(false);
-                    error_message.get_or_insert_with(|| format!("Failed to sync ID lists: {e}"));
-                }
-            }
-            if let Err(e) = adapter
-                .clone()
-                .schedule_background_sync(&self.statsig_runtime)
-                .await
-            {
-                success = false;
-                log_w!(TAG, "Failed to schedule idlist background job {}", e);
-            }
-        }
+        self.log_init_details(&init_details);
 
-        self.event_logging_adapter
-            .clone()
-            .start(&self.statsig_runtime)
-            .await
-            .map_err(|e| {
-                success = false;
-                log_error_to_statsig_and_console!(
-                    self.ops_stats.clone(),
-                    TAG,
-                    "Failed to start event logging adaper {}",
-                    e
-                );
-                e
-            })?;
-
-        if let Err(e) = self
-            .specs_adapter
-            .clone()
-            .schedule_background_sync(&self.statsig_runtime)
-            .await
-        {
-            success = false;
-            log_error_to_statsig_and_console!(
-                self.ops_stats,
-                TAG,
-                "Failed to schedule SpecAdapter({}) background job. Error: {}",
-                self.specs_adapter.get_type_name(),
-                e,
+        init_details.and_then(|details| {
+            // Touch all fields to avoid dead code warnings - will use these properly in the future
+            let _ = (
+                details.duration,
+                details.init_success,
+                details.is_config_spec_ready,
+                details.is_id_list_ready,
+                details.source,
             );
-        }
 
-        let spec_info = self.spec_store.get_current_specs_info();
-        let duration = start_time.elapsed().as_millis() as f64;
-
-        self.set_default_environment_from_server();
-        self.log_init_finish(success, &error_message, &duration, &spec_info);
-        let error = init_res.clone().err();
-        if let Some(ref e) = error {
-            log_error_to_statsig_and_console!(
-                self.ops_stats,
-                TAG,
-                "{}",
-                error_message.clone().unwrap_or(e.to_string())
-            );
-        }
-
-        Ok(StatsigInitializeDetails {
-            init_success: success,
-            is_config_spec_ready: spec_info.lcut.is_some(),
-            is_id_list_ready: id_list_ready,
-            source: spec_info.source,
-            failure_details: error.as_ref().map(|e| FailureDetails {
-                reason: e.to_string(),
-                error: Some(e.clone()),
-            }),
-            duration,
+            if let Some(failure_details) = details.failure_details {
+                Err(failure_details
+                    .error
+                    .unwrap_or(StatsigErr::InitializationError(failure_details.reason)))
+            } else {
+                Ok(())
+            }
         })
     }
 
@@ -331,12 +254,6 @@ impl Statsig {
     }
 
     pub async fn shutdown_with_timeout(&self, timeout: Duration) -> Result<(), StatsigErr> {
-        log_d!(
-            TAG,
-            "Shutting down Statsig with timeout {}ms",
-            timeout.as_millis()
-        );
-
         let start = Instant::now();
         let final_result = self.__shutdown_internal(timeout).await;
         self.finalize_shutdown(timeout.saturating_sub(start.elapsed()));
@@ -419,6 +336,248 @@ impl Statsig {
         self.statsig_runtime.shutdown(timeout);
     }
 
+    async fn start_background_tasks(
+        event_logger: Arc<EventLogger>,
+        statsig_runtime: Arc<StatsigRuntime>,
+        id_lists_adapter: Option<Arc<dyn IdListsAdapter>>,
+        specs_adapter: Arc<dyn SpecsAdapter>,
+        ops_stats: Arc<OpsStatsForInstance>,
+        bg_tasks_started: Arc<AtomicBool>,
+    ) -> bool {
+        if bg_tasks_started.load(Ordering::SeqCst) {
+            return true;
+        }
+
+        let mut success = true;
+        event_logger.clone().start_background_task(&statsig_runtime);
+
+        if let Some(adapter) = &id_lists_adapter {
+            if let Err(e) = adapter
+                .clone()
+                .schedule_background_sync(&statsig_runtime)
+                .await
+            {
+                success = false;
+                log_w!(TAG, "Failed to schedule idlist background job {}", e);
+            }
+        }
+
+        if let Err(e) = specs_adapter
+            .clone()
+            .schedule_background_sync(&statsig_runtime)
+            .await
+        {
+            success = false;
+            log_error_to_statsig_and_console!(
+                ops_stats,
+                TAG,
+                "Failed to schedule SpecAdapter({}) background job. Error: {}",
+                specs_adapter.get_type_name(),
+                e,
+            );
+        }
+
+        bg_tasks_started.store(true, Ordering::SeqCst);
+
+        success
+    }
+
+    async fn apply_timeout_to_init(
+        &self,
+        timeout_ms: u64,
+    ) -> Result<StatsigInitializeDetails, StatsigErr> {
+        let timeout = Duration::from_millis(timeout_ms);
+
+        let init_future = self.initialize_impl_with_details();
+        let timeout_future = sleep(timeout);
+
+        let event_logger = self.event_logger.clone();
+        let statsig_runtime = self.statsig_runtime.clone();
+        let id_lists_adapter = self.id_lists_adapter.clone();
+        let specs_adapter = self.specs_adapter.clone();
+        let ops_stats = self.ops_stats.clone();
+        let background_tasks_started = self.background_tasks_started.clone();
+        // Create another clone specifically for the closure
+        let statsig_runtime_for_closure = statsig_runtime.clone();
+
+        tokio::select! {
+            result = init_future => {
+                result
+            },
+            _ = timeout_future => {
+                statsig_runtime.spawn(
+                    "start_background_tasks",
+                    |_shutdown_notify| async move {
+                        Self::start_background_tasks(
+                            event_logger,
+                            statsig_runtime_for_closure,
+                            id_lists_adapter,
+                            specs_adapter,
+                            ops_stats,
+                            background_tasks_started,
+                        ).await;
+                    }
+                );
+                Ok(self.timeout_failure(timeout_ms))
+            },
+        }
+    }
+
+    async fn initialize_impl_with_details(&self) -> Result<StatsigInitializeDetails, StatsigErr> {
+        let start_time = Instant::now();
+        self.spec_store.set_source(SpecsSource::Loading);
+        self.specs_adapter.initialize(self.spec_store.clone());
+
+        let mut error_message = None;
+        let mut id_list_ready = None;
+
+        let init_country_lookup = if !self.options.disable_country_lookup.unwrap_or_default() {
+            Some(self.statsig_runtime.spawn(INIT_IP_TAG, |_| async {
+                CountryLookup::load_country_lookup();
+            }))
+        } else {
+            None
+        };
+
+        let init_ua = if !self.options.disable_user_agent_parsing.unwrap_or_default() {
+            Some(self.statsig_runtime.spawn(INIT_UA_TAG, |_| async {
+                UserAgentParser::load_parser();
+            }))
+        } else {
+            None
+        };
+
+        let init_res = match self
+            .specs_adapter
+            .clone()
+            .start(&self.statsig_runtime)
+            .await
+        {
+            Ok(()) => Ok(()),
+            Err(e) => {
+                self.spec_store.set_source(SpecsSource::NoValues);
+                error_message = Some(format!("Failed to start specs adapter: {e}"));
+                Err(e)
+            }
+        };
+
+        if let Some(adapter) = &self.id_lists_adapter {
+            match adapter
+                .clone()
+                .start(&self.statsig_runtime, self.spec_store.clone())
+                .await
+            {
+                Ok(()) => {
+                    id_list_ready = Some(true);
+                }
+                Err(e) => {
+                    id_list_ready = Some(false);
+                    error_message.get_or_insert_with(|| format!("Failed to sync ID lists: {e}"));
+                }
+            }
+            if let Err(e) = adapter
+                .clone()
+                .schedule_background_sync(&self.statsig_runtime)
+                .await
+            {
+                log_w!(TAG, "Failed to schedule id_list background job {}", e);
+            }
+        }
+
+        if let Err(e) = self
+            .event_logging_adapter
+            .clone()
+            .start(&self.statsig_runtime)
+            .await
+        {
+            log_error_to_statsig_and_console!(
+                self.ops_stats.clone(),
+                TAG,
+                "Failed to start event logging adapter {}",
+                e
+            );
+        }
+
+        let spec_info = self.spec_store.get_current_specs_info();
+        let duration = start_time.elapsed().as_millis() as f64;
+
+        self.set_default_environment_from_server();
+
+        if self.options.wait_for_country_lookup_init.unwrap_or(false) {
+            if let Some(task_id) = init_country_lookup {
+                let _ = self
+                    .statsig_runtime
+                    .await_join_handle(INIT_IP_TAG, &task_id)
+                    .await;
+            }
+        }
+        if self.options.wait_for_user_agent_init.unwrap_or(false) {
+            if let Some(task_id) = init_ua {
+                let _ = self
+                    .statsig_runtime
+                    .await_join_handle(INIT_UA_TAG, &task_id)
+                    .await;
+            };
+        }
+
+        let error = init_res.clone().err();
+
+        let success = Self::start_background_tasks(
+            self.event_logger.clone(),
+            self.statsig_runtime.clone(),
+            self.id_lists_adapter.clone(),
+            self.specs_adapter.clone(),
+            self.ops_stats.clone(),
+            self.background_tasks_started.clone(),
+        )
+        .await;
+
+        Ok(StatsigInitializeDetails {
+            init_success: success,
+            is_config_spec_ready: spec_info.lcut.is_some(),
+            is_id_list_ready: id_list_ready,
+            source: spec_info.source,
+            failure_details: error.as_ref().map(|e| FailureDetails {
+                reason: e.to_string(),
+                error: Some(e.clone()),
+            }),
+            duration,
+        })
+    }
+
+    fn timeout_failure(&self, timeout_ms: u64) -> StatsigInitializeDetails {
+        StatsigInitializeDetails {
+            init_success: false,
+            is_config_spec_ready: false,
+            is_id_list_ready: None,
+            source: SpecsSource::Uninitialized,
+            failure_details: Some(FailureDetails {
+                reason: "Initialization timed out".to_string(),
+                error: None,
+            }),
+            duration: timeout_ms as f64,
+        }
+    }
+
+    fn log_init_details(&self, init_details: &Result<StatsigInitializeDetails, StatsigErr>) {
+        match init_details {
+            Ok(details) => {
+                self.log_init_finish(
+                    details.init_success,
+                    &None,
+                    &details.duration,
+                    &self.spec_store.get_current_specs_info(),
+                );
+                if let Some(failure) = &details.failure_details {
+                    log_error_to_statsig_and_console!(self.ops_stats, TAG, "{}", failure.reason);
+                }
+            }
+            Err(err) => {
+                log_w!(TAG, "Initialization error: {:?}", err);
+            }
+        }
+    }
+
     pub fn get_context(&self) -> StatsigContext {
         StatsigContext {
             sdk_key: self.sdk_key.clone(),
@@ -448,7 +607,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(QueuedEventPayload::CustomEvent(make_custom_event(
-                user_internal,
+                user_internal.to_loggable(),
                 StatsigEvent {
                     event_name: event_name.to_string(),
                     value: value.map(|v| json!(v)),
@@ -469,7 +628,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(QueuedEventPayload::CustomEvent(make_custom_event(
-                user_internal,
+                user_internal.to_loggable(),
                 StatsigEvent {
                     event_name: event_name.to_string(),
                     value: value.map(|v| json!(v)),
@@ -510,7 +669,7 @@ impl Statsig {
         let layer_eval = layer.__evaluation.as_ref();
 
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &layer.__user,
+            &layer.__user.get_sampling_key(),
             layer_eval.map(AnyEvaluation::from).as_ref(),
             Some(&parameter_name),
         );
@@ -534,7 +693,7 @@ impl Statsig {
     }
 
     pub async fn flush_events(&self) {
-        self.event_logger.flush_blocking().await;
+        self.event_logger.flush_all_blocking().await;
     }
 
     pub fn get_client_init_response(&self, user: &StatsigUser) -> InitializeResponse {
@@ -763,10 +922,11 @@ impl Statsig {
             group_name: None,
             is_experiment_active: Some(true),
             is_user_in_experiment: Some(true),
+            undelegated_secondary_exposures: None,
         };
 
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &user_internal,
+            &user_internal.get_sampling_key(),
             Some(&AnyEvaluation::from(&experiment_eval)),
             None,
         );
@@ -777,7 +937,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(QueuedEventPayload::ConfigExposure(ConfigExposure {
-                user: user_internal,
+                user: user_internal.to_loggable(),
                 evaluation: Some(base_eval),
                 evaluation_details: experiment.details.clone(),
                 config_name: cmab_name.to_string(),
@@ -888,10 +1048,7 @@ impl Statsig {
         gate_name: &str,
         options: FeatureGateEvaluationOptions,
     ) -> FeatureGate {
-        log_d!(TAG, "Get Feature Gate {}", gate_name);
-
         let user_internal = self.internalize_user(user);
-
         let disable_exposure_logging = options.disable_exposure_logging;
         let gate = self.get_feature_gate_impl(&user_internal, gate_name);
 
@@ -1082,7 +1239,12 @@ impl Statsig {
     ) -> Experiment {
         let user_internal = self.internalize_user(user);
         let disable_exposure_logging = options.disable_exposure_logging;
-        let experiment = self.get_experiment_impl(&user_internal, experiment_name);
+        let mut experiment = self.get_experiment_impl(&user_internal, experiment_name);
+        if let Some(persisted_experiment) = self.persistent_values_manager.as_ref().and_then(|m| {
+            m.try_apply_sticky_value_to_experiment(&user_internal, &options, &experiment)
+        }) {
+            experiment = persisted_experiment
+        }
 
         if disable_exposure_logging {
             log_d!(
@@ -1142,7 +1304,7 @@ impl Statsig {
         options: LayerEvaluationOptions,
     ) -> Layer {
         let user_internal = self.internalize_user(user);
-        self.get_layer_impl(&user_internal, layer_name, options)
+        self.get_layer_impl(user_internal, layer_name, options)
     }
 
     pub fn manually_log_layer_parameter_exposure(
@@ -1152,16 +1314,13 @@ impl Statsig {
         parameter_name: String,
     ) {
         let user_internal = self.internalize_user(user);
-        let layer = self.get_layer_impl(
-            &user_internal,
-            layer_name,
-            LayerEvaluationOptions::default(),
-        );
+        let layer =
+            self.get_layer_impl(user_internal, layer_name, LayerEvaluationOptions::default());
 
         let layer_eval = layer.__evaluation.as_ref();
 
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &layer.__user,
+            &layer.__user.get_sampling_key(),
             layer_eval.map(AnyEvaluation::from).as_ref(),
             Some(parameter_name.as_str()),
         );
@@ -1205,6 +1364,58 @@ impl Statsig {
     }
 }
 
+// -------------------------
+//   Internal Functions
+// -------------------------
+
+impl Statsig {
+    pub(crate) fn get_from_statsig_env(&self, key: &str) -> Option<DynamicValue> {
+        if let Some(env) = &self.statsig_environment {
+            return env.get(key).cloned();
+        }
+
+        if let Ok(fallback_env) = self.fallback_environment.lock() {
+            if let Some(env) = &*fallback_env {
+                return env.get(key).cloned();
+            }
+        }
+
+        None
+    }
+
+    pub(crate) fn get_value_from_global_custom_fields(&self, key: &str) -> Option<&DynamicValue> {
+        if let Some(env) = &self.options.global_custom_fields {
+            return env.get(key);
+        }
+
+        None
+    }
+
+    pub(crate) fn use_global_custom_fields<T>(
+        &self,
+        f: impl FnOnce(Option<&HashMap<String, DynamicValue>>) -> Result<(), T>,
+    ) -> Result<(), T> {
+        f(self.options.global_custom_fields.as_ref())
+    }
+
+    pub(crate) fn use_statsig_env<T>(
+        &self,
+        f: impl FnOnce(Option<&HashMap<String, DynamicValue>>) -> T,
+    ) -> T {
+        if let Some(env) = &self.statsig_environment {
+            return f(Some(env));
+        }
+
+        if let Ok(fallback_env) = self.fallback_environment.lock() {
+            if let Some(env) = &*fallback_env {
+                return f(Some(env));
+            }
+        }
+
+        f(None)
+    }
+}
+
 // -------------------------
 //   Private Functions
 // -------------------------
@@ -1329,22 +1540,25 @@ impl Statsig {
 
     fn get_layer_impl(
         &self,
-        user_internal: &StatsigUserInternal,
+        user_internal: StatsigUserInternal,
         layer_name: &str,
         evaluation_options: LayerEvaluationOptions,
     ) -> Layer {
         let disable_exposure_logging = evaluation_options.disable_exposure_logging;
+        let event_logger_ptr = Arc::downgrade(&self.event_logger);
+        let sampling_processor_ptr = Arc::downgrade(&self.sampling_processor);
+
         if disable_exposure_logging {
             self.event_logger
                 .increment_non_exposure_checks_count(layer_name.to_string());
         }
 
-        self.evaluate_spec(
-            user_internal,
+        let mut layer = self.evaluate_spec(
+            &user_internal,
             layer_name,
             |eval_details| {
                 make_layer(
-                    user_internal,
+                    user_internal.to_loggable(),
                     layer_name,
                     None,
                     eval_details,
@@ -1361,7 +1575,7 @@ impl Statsig {
                 let sampling_processor_ptr = Arc::downgrade(&self.sampling_processor);
 
                 make_layer(
-                    user_internal,
+                    user_internal.to_loggable(),
                     layer_name,
                     Some(evaluation),
                     eval_details,
@@ -1373,21 +1587,33 @@ impl Statsig {
                 )
             },
             &SpecType::Layer,
-        )
+        );
+        if let Some(persisted_layer) = self.persistent_values_manager.as_ref().and_then(|p| {
+            p.try_apply_sticky_value_to_layer(
+                &user_internal,
+                &evaluation_options,
+                &layer,
+                Some(event_logger_ptr),
+                Some(sampling_processor_ptr),
+                disable_exposure_logging,
+            )
+        }) {
+            layer = persisted_layer
+        }
+        layer
     }
 
     fn log_gate_exposure(
         &self,
-        user_internal: StatsigUserInternal,
+        user: StatsigUserInternal,
         gate_name: &str,
         gate: &FeatureGate,
         is_manual: bool,
     ) {
         let gate_eval = gate.__evaluation.as_ref();
         let secondary_exposures = gate_eval.map(|eval| &eval.base.secondary_exposures);
-
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &user_internal,
+            &user.get_sampling_key(),
             gate_eval.map(AnyEvaluation::from).as_ref(),
             None,
         );
@@ -1398,7 +1624,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(QueuedEventPayload::GateExposure(GateExposure {
-                user: user_internal,
+                user: user.to_loggable(),
                 gate_name: gate_name.to_string(),
                 value: gate.value,
                 rule_id: Some(gate.rule_id.clone()),
@@ -1420,9 +1646,10 @@ impl Statsig {
     ) {
         let config_eval = dynamic_config.__evaluation.as_ref();
         let base_eval = config_eval.map(|eval| eval.base.clone());
+        let loggable_user = user_internal.to_loggable();
 
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &user_internal,
+            &loggable_user.get_sampling_key(),
             config_eval.map(AnyEvaluation::from).as_ref(),
             None,
         );
@@ -1433,7 +1660,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(QueuedEventPayload::ConfigExposure(ConfigExposure {
-                user: user_internal,
+                user: loggable_user,
                 evaluation: base_eval,
                 evaluation_details: dynamic_config.details.clone(),
                 config_name: dynamic_config_name.to_string(),
@@ -1456,7 +1683,7 @@ impl Statsig {
         let base_eval = experiment_eval.map(|eval| eval.base.clone());
 
         let sampling_details = self.sampling_processor.get_sampling_decision_and_details(
-            &user_internal,
+            &user_internal.get_sampling_key(),
             experiment_eval.map(AnyEvaluation::from).as_ref(),
             None,
         );
@@ -1465,9 +1692,11 @@ impl Statsig {
             return;
         }
 
+        let loggable_user = user_internal.to_loggable();
+
         self.event_logger
             .enqueue(QueuedEventPayload::ConfigExposure(ConfigExposure {
-                user: user_internal,
+                user: loggable_user,
                 evaluation: base_eval,
                 evaluation_details: experiment.details.clone(),
                 config_name: experiment_name.to_string(),
@@ -1479,26 +1708,8 @@ impl Statsig {
             }));
     }
 
-    fn internalize_user(&self, user: &StatsigUser) -> StatsigUserInternal {
-        StatsigUserInternal::new(
-            user,
-            self.get_statsig_env(),
-            self.options.global_custom_fields.clone(),
-        )
-    }
-
-    fn get_statsig_env(&self) -> Option<HashMap<String, DynamicValue>> {
-        if let Some(env) = &self.statsig_environment {
-            return Some(env.clone());
-        }
-
-        if let Ok(fallback_env) = self.fallback_environment.lock() {
-            if let Some(env) = &*fallback_env {
-                return Some(env.clone());
-            }
-        }
-
-        None
+    fn internalize_user<'s, 'u>(&'s self, user: &'u StatsigUser) -> StatsigUserInternal<'s, 'u> {
+        StatsigUserInternal::new(user, Some(self))
     }
 
     fn set_default_environment_from_server(&self) {
@@ -1649,111 +1860,3 @@ fn setup_ops_stats(
 
     ops_stat
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::hashing::djb2;
-    use crate::{
-        evaluation::evaluation_types::AnyConfigEvaluation, output_logger::LogLevel,
-        StatsigHttpIdListsAdapter,
-    };
-    use std::env;
-
-    impl Statsig {}
-
-    fn get_sdk_key() -> String {
-        let key = env::var("test_api_key").expect("test_api_key environment variable not set");
-        assert!(key.starts_with("secret-9IWf"));
-        key
-    }
-
-    #[tokio::test]
-    async fn test_check_gate() {
-        let user = StatsigUser {
-            email: Some(dyn_value!("daniel@statsig.com")),
-            ..StatsigUser::with_user_id("a-user".to_string())
-        };
-
-        let statsig = Statsig::new(&get_sdk_key(), None);
-        statsig.initialize().await.unwrap();
-
-        let gate_result = statsig.check_gate(&user, "test_50_50");
-
-        assert!(gate_result);
-    }
-
-    #[tokio::test]
-    async fn test_check_gate_id_list() {
-        let user = StatsigUser {
-            custom_ids: Some(HashMap::from([(
-                "companyID".to_string(),
-                dyn_value!("marcos_1"),
-            )])),
-            ..StatsigUser::with_user_id("marcos_1".to_string())
-        };
-
-        let mut opts = StatsigOptions::new();
-
-        let adapter = Arc::new(StatsigHttpIdListsAdapter::new(&get_sdk_key(), &opts));
-        opts.id_lists_adapter = Some(adapter);
-
-        let statsig = Statsig::new(&get_sdk_key(), Some(Arc::new(opts)));
-        statsig.initialize().await.unwrap();
-
-        let gate_result = statsig.check_gate(&user, "test_id_list");
-
-        assert!(gate_result);
-    }
-
-    #[tokio::test]
-    async fn test_get_experiment() {
-        let user = StatsigUser {
-            email: Some(dyn_value!("daniel@statsig.com")),
-            ..StatsigUser::with_user_id("a-user".to_string())
-        };
-
-        let statsig = Statsig::new(&get_sdk_key(), None);
-        statsig.initialize().await.unwrap();
-
-        let experiment = statsig.get_experiment(&user, "running_exp_in_unlayered_with_holdout");
-        let _ = statsig.shutdown().await;
-
-        assert_ne!(experiment.value.len(), 0);
-    }
-
-    #[tokio::test]
-    async fn test_gcir() {
-        let user = StatsigUser {
-            email: Some(dyn_value!("daniel@statsig.com")),
-            ..StatsigUser::with_user_id("a-user".to_string())
-        };
-        let opts = StatsigOptions {
-            output_log_level: Some(LogLevel::Debug),
-            ..StatsigOptions::new()
-        };
-
-        let statsig = Statsig::new(&get_sdk_key(), Some(Arc::new(opts)));
-        statsig.initialize().await.unwrap();
-
-        let response = statsig.get_client_init_response(&user);
-        let _ = statsig.shutdown().await;
-
-        let gates = response.feature_gates;
-        assert_eq!(gates.len(), 69);
-
-        let configs = response.dynamic_configs.len();
-        assert_eq!(configs, 62);
-
-        let a_config_opt = response.dynamic_configs.get(&djb2("big_number"));
-        let a_config = match a_config_opt {
-            Some(v) => match v {
-                AnyConfigEvaluation::DynamicConfig(config) => &config.value,
-                AnyConfigEvaluation::Experiment(exp) => &exp.value,
-            },
-            None => panic!("Should have values"),
-        };
-
-        assert!(!a_config.is_empty());
-    }
-}
