@@ -6,6 +6,7 @@ use crate::evaluation::dynamic_value::DynamicValue;
 use crate::event_logging_adapter::EventLoggingAdapter;
 use crate::id_lists_adapter::IdListsAdapter;
 use crate::output_logger::LogLevel;
+use crate::persistent_storage::persistent_storage_trait::PersistentStorage;
 use crate::{
     serialize_if_not_none, ObservabilityClient, OverrideAdapter, SpecAdapterConfig, SpecsAdapter,
 };
@@ -18,19 +19,22 @@ pub const DEFAULT_INIT_TIMEOUT_MS: u64 = 3000;
 #[derive(Clone, Default)]
 pub struct StatsigOptions {
     pub data_store: Option<Arc<dyn DataStoreTrait>>, // External DataStore
-    pub enable_id_lists: Option<bool>,
-    pub enable_user_agent_parsing: Option<bool>,
-    pub enable_country_lookup: Option<bool>,
-    pub environment: Option<String>,
 
-    pub disable_network: Option<bool>, // Disable all out-going network including get configs, log_events...
     pub disable_all_logging: Option<bool>,
+    pub disable_country_lookup: Option<bool>,
+    pub disable_network: Option<bool>, // Disable all out-going network including get configs, log_events...
+    pub disable_user_agent_parsing: Option<bool>,
+
+    pub enable_id_lists: Option<bool>,
+    pub environment: Option<String>,
 
     pub event_logging_adapter: Option<Arc<dyn EventLoggingAdapter>>,
     pub event_logging_flush_interval_ms: Option<u32>,
+    pub event_logging_max_pending_batch_queue_size: Option<u32>,
     pub event_logging_max_queue_size: Option<u32>,
 
     pub fallback_to_statsig_api: Option<bool>,
+    pub global_custom_fields: Option<HashMap<String, DynamicValue>>,
 
     pub id_lists_adapter: Option<Arc<dyn IdListsAdapter>>,
     pub id_lists_sync_interval_ms: Option<u32>,
@@ -41,13 +45,16 @@ pub struct StatsigOptions {
     pub observability_client: Option<Weak<dyn ObservabilityClient>>,
     pub output_log_level: Option<LogLevel>,
     pub override_adapter: Option<Arc<dyn OverrideAdapter>>,
+    pub persistent_storage: Option<Arc<dyn PersistentStorage>>,
+    pub service_name: Option<String>,
 
     pub spec_adapters_config: Option<Vec<SpecAdapterConfig>>, // Specs to customized spec adapter, order matters, reflecting priority of trying
     pub specs_adapter: Option<Arc<dyn SpecsAdapter>>,
     pub specs_sync_interval_ms: Option<u32>,
     pub specs_url: Option<String>,
-    pub service_name: Option<String>,
-    pub global_custom_fields: Option<HashMap<String, DynamicValue>>,
+
+    pub wait_for_country_lookup_init: Option<bool>,
+    pub wait_for_user_agent_init: Option<bool>,
 }
 
 impl StatsigOptions {
@@ -118,6 +125,9 @@ impl StatsigOptionsBuilder {
     }
 
     #[must_use]
+    #[deprecated(
+        note = "This field is deprecated in favor of smart log event. It is no longer consumed and can be removed safely."
+    )]
     pub fn event_logging_flush_interval_ms(
         mut self,
         event_logging_flush_interval_ms: Option<u32>,
@@ -135,6 +145,16 @@ impl StatsigOptionsBuilder {
         self
     }
 
+    #[must_use]
+    pub fn event_logging_max_pending_batch_queue_size(
+        mut self,
+        event_logging_max_pending_batch_queue_size: Option<u32>,
+    ) -> Self {
+        self.inner.event_logging_max_pending_batch_queue_size =
+            event_logging_max_pending_batch_queue_size;
+        self
+    }
+
     // ID Lists
 
     #[must_use]
@@ -177,6 +197,33 @@ impl StatsigOptionsBuilder {
         self
     }
 
+    #[must_use]
+    pub fn wait_for_country_lookup_init(
+        mut self,
+        wait_for_country_lookup_init: Option<bool>,
+    ) -> Self {
+        self.inner.wait_for_country_lookup_init = wait_for_country_lookup_init;
+        self
+    }
+
+    #[must_use]
+    pub fn wait_for_user_agent_init(mut self, wait_for_user_agent_init: Option<bool>) -> Self {
+        self.inner.wait_for_user_agent_init = wait_for_user_agent_init;
+        self
+    }
+
+    #[must_use]
+    pub fn disable_user_agent_parsing(mut self, disable_user_agent_parsing: Option<bool>) -> Self {
+        self.inner.disable_user_agent_parsing = disable_user_agent_parsing;
+        self
+    }
+
+    #[must_use]
+    pub fn disable_country_lookup(mut self, disable_country_lookup: Option<bool>) -> Self {
+        self.inner.disable_country_lookup = disable_country_lookup;
+        self
+    }
+
     #[must_use]
     pub fn service_name(mut self, service_name: Option<String>) -> Self {
         self.inner.service_name = service_name;
@@ -189,18 +236,6 @@ impl StatsigOptionsBuilder {
         self
     }
 
-    #[must_use]
-    pub fn enable_user_agent_parsing(mut self, enable_user_agent_parsing: Option<bool>) -> Self {
-        self.inner.enable_user_agent_parsing = enable_user_agent_parsing;
-        self
-    }
-
-    #[must_use]
-    pub fn enable_country_lookup(mut self, enable_country_lookup: Option<bool>) -> Self {
-        self.inner.enable_country_lookup = enable_country_lookup;
-        self
-    }
-
     #[must_use]
     pub fn global_custom_fields(
         mut self,
@@ -216,8 +251,8 @@ impl StatsigOptionsBuilder {
     }
 
     #[must_use]
-    pub fn observability_client(mut self, client: Option<Weak<dyn ObservabilityClient>>) -> Self {
-        self.inner.observability_client = client;
+    pub fn init_timeout_ms(mut self, init_timeout_ms: Option<u64>) -> Self {
+        self.inner.init_timeout_ms = init_timeout_ms;
         self
     }
 
@@ -225,6 +260,20 @@ impl StatsigOptionsBuilder {
     pub fn build(self) -> StatsigOptions {
         self.inner
     }
+
+    // interface related options
+
+    #[must_use]
+    pub fn observability_client(mut self, client: Option<Weak<dyn ObservabilityClient>>) -> Self {
+        self.inner.observability_client = client;
+        self
+    }
+
+    #[must_use]
+    pub fn data_store(mut self, data_store: Option<Arc<dyn DataStoreTrait>>) -> Self {
+        self.inner.data_store = data_store;
+        self
+    }
 }
 
 impl Serialize for StatsigOptions {
@@ -257,10 +306,14 @@ impl Serialize for StatsigOptions {
         serialize_if_not_none!(state, "enable_id_lists", &self.enable_id_lists);
         serialize_if_not_none!(
             state,
-            "enable_user_agent_parsing",
-            &self.enable_user_agent_parsing
+            "wait_for_user_agent_init",
+            &self.wait_for_user_agent_init
+        );
+        serialize_if_not_none!(
+            state,
+            "wait_for_country_lookup_init",
+            &self.wait_for_country_lookup_init
         );
-        serialize_if_not_none!(state, "enable_country_lookup", &self.enable_country_lookup);
         serialize_if_not_none!(
             state,
             "id_lists_sync_interval",
